from aspy_dependency_injection.abstractions.base_service_provider import (
    BaseServiceProvider,
)


class ObjectDisposedError(Exception):
    """The exception that is thrown when an operation is performed on a disposed object."""

    def __init__(self) -> None:
        super().__init__(BaseServiceProvider.__name__)



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from collections.abc import Awaitable, Callable

    from aspy_dependency_injection._service_lookup._service_call_site import (
        ServiceCallSite,
    )
    from aspy_dependency_injection.service_provider_engine_scope import (
        ServiceProviderEngineScope,
    )


class ServiceProviderEngine(ABC):
    @abstractmethod
    def realize_service(
        self, call_site: ServiceCallSite
    ) -> Callable[[ServiceProviderEngineScope], Awaitable[object | None]]: ...



from collections.abc import Hashable
from typing import TYPE_CHECKING, Final, final, override

if TYPE_CHECKING:
    from aspy_dependency_injection._service_lookup._service_identifier import (
        ServiceIdentifier,
    )


@final
class ServiceCacheKey(Hashable):
    _service_identifier: Final[ServiceIdentifier]
    _slot: Final[int]

    def __init__(self, service_identifier: ServiceIdentifier, slot: int) -> None:
        self._service_identifier = service_identifier
        self._slot = slot

    @property
    def service_identifier(self) -> ServiceIdentifier:
        return self._service_identifier

    @property
    def slot(self) -> int:
        return self._slot

    @override
    def __hash__(self) -> int:
        return (hash(self._service_identifier) * 397) ^ self._slot

    def __eq__(self, value: object) -> bool:
        if not isinstance(value, ServiceCacheKey):
            return NotImplemented

        return (
            self._service_identifier == value.service_identifier
            and self._slot == value.slot
        )



from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from aspy_dependency_injection.abstractions.service_scope import ServiceScope


class ServiceScopeFactory(ABC):
    @abstractmethod
    def create_scope(self) -> ServiceScope: ...



from typing import TYPE_CHECKING, Final, Self

from aspy_dependency_injection._service_lookup._typed_type import TypedType

if TYPE_CHECKING:
    from collections.abc import Awaitable, Callable

    from aspy_dependency_injection.service_lifetime import ServiceLifetime


class ServiceDescriptor:
    """Service registration."""

    _service_type: Final[TypedType]
    _lifetime: Final[ServiceLifetime]
    _implementation_type: TypedType | None
    _sync_implementation_factory: Callable[..., object] | None
    _async_implementation_factory: Callable[..., Awaitable[object]] | None

    def __init__(self, service_type: type, lifetime: ServiceLifetime) -> None:
        self._service_type = TypedType.from_type(service_type)
        self._lifetime = lifetime
        self._implementation_type = None
        self._sync_implementation_factory = None
        self._async_implementation_factory = None

    @property
    def service_type(self) -> TypedType:
        return self._service_type

    @property
    def lifetime(self) -> ServiceLifetime:
        return self._lifetime

    @property
    def implementation_type(self) -> TypedType | None:
        return self._implementation_type

    @property
    def sync_implementation_factory(
        self,
    ) -> Callable[..., object] | None:
        return self._sync_implementation_factory

    @property
    def async_implementation_factory(
        self,
    ) -> Callable[..., Awaitable[object]] | None:
        return self._async_implementation_factory

    @classmethod
    def from_implementation_type(
        cls, service_type: type, implementation_type: type, lifetime: ServiceLifetime
    ) -> Self:
        self = cls(service_type=service_type, lifetime=lifetime)
        self._implementation_type = TypedType.from_type(implementation_type)
        return self

    @classmethod
    def from_sync_implementation_factory(
        cls,
        service_type: type,
        implementation_factory: Callable[..., object],
        lifetime: ServiceLifetime,
    ) -> Self:
        self = cls(service_type=service_type, lifetime=lifetime)
        self._sync_implementation_factory = implementation_factory
        return self

    @classmethod
    def from_async_implementation_factory(
        cls,
        service_type: type,
        implementation_factory: Callable[..., Awaitable[object]],
        lifetime: ServiceLifetime,
    ) -> Self:
        self = cls(service_type=service_type, lifetime=lifetime)
        self._async_implementation_factory = implementation_factory
        return self

    def has_implementation_type(self) -> bool:
        return self._implementation_type is not None



from fastapi import Depends

from aspy_dependency_injection.injectable import Injectable


def Inject() -> Injectable:  # noqa: N802
    """Inject :func:`Depends` for FastAPI integration."""

    def _dependency() -> Injectable:
        return Injectable()

    return Depends(_dependency, use_cache=False)
